<?php

declare(strict_types=1);

namespace Dedi\SyliusCustomerReferralPlugin\Application\Controller\Shop;

use Dedi\SyliusCustomerReferralPlugin\Application\Form\Shop\Type\ReferralInviteType;
use Dedi\SyliusCustomerReferralPlugin\Domain\Referral\UseCase\Invite\Invite;
use Dedi\SyliusCustomerReferralPlugin\Domain\Referral\UseCase\Invite\InviteInput;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Contracts\Translation\TranslatorInterface;

final class ReferralInviteController extends AbstractController
{
    /** @var TranslatorInterface */
    private $translator;

    /** @var RequestStack */
    private $requestStack;

    /** @var Invite */
    private $invite;

    public function __construct(
        TranslatorInterface $translator,
        RequestStack $requestStack,
        Invite $invite
    ) {
        $this->translator = $translator;
        $this->requestStack = $requestStack;
        $this->invite = $invite;
    }

    public function __invoke(): Response
    {
        $request = $this->requestStack->getMasterRequest();

        $form = $this->createForm(ReferralInviteType::class, InviteInput::createWithEmails());
        $form->handleRequest($request);

        if ($form->isSubmitted()) {
            if (!$form->isValid()) {
                foreach ($form->getErrors(true) as $error) {
                    $this->addFlash('error', $error->getMessage());
                }

                return $this->redirectToRoute('shop_account_referral_index');
            }

            $presenter = new ReferralInvitePresenter();
            ($this->invite)($form->getData(), $presenter);

            if ($emails = $presenter->getNewInviteeEmails()) {
                $this->addFlash('success', $this->translator->trans('dedi_customer_referral.ui.invitees_invited', [
                    '%invitees%' => implode(', ', $emails),
                ], 'flashes'));
            }

            if ($emails = $presenter->getAlreadyInvitedInviteeEmails()) {
                $this->addFlash('error', $this->translator->trans('dedi_customer_referral.ui.invitees_already_invited', [
                    '%invitees%' => implode(', ', $emails),
                ], 'flashes'));
            }

            return $this->redirectToRoute('shop_account_referral_index');
        }

        return $this->render('@DediSyliusCustomerReferralPlugin/Shop/Referral/_invite.html.twig', [
            'form' => $form->createView(),
        ]);
    }
}
