<?php

declare(strict_types=1);

namespace Dedi\SyliusCustomerReferralPlugin\Domain\Referral\UseCase\AcceptInvite;

use Dedi\SyliusCustomerReferralPlugin\Domain\Referral\Exception\InviteeAlreadyInvitedException;
use Dedi\SyliusCustomerReferralPlugin\Domain\Referral\Exception\ReferralAlreadyAcceptedException;
use Dedi\SyliusCustomerReferralPlugin\Domain\Referral\Exception\ReferralNotFoundForInviteeEmail;
use Dedi\SyliusCustomerReferralPlugin\Domain\Referral\Repository\ReferralRepositoryInterface;
use Dedi\SyliusCustomerReferralPlugin\Domain\Referral\Specification\IsReferralStillAvailableSpecification;

class AcceptInvite
{
    /** @var ReferralRepositoryInterface */
    private $referralRepository;

    /** @var IsReferralStillAvailableSpecification */
    private $isReferralStillAvailableSpecification;

    public function __construct(
        ReferralRepositoryInterface $referralRepository,
        IsReferralStillAvailableSpecification $isReferralStillAvailableSpecification
    ) {
        $this->referralRepository = $referralRepository;
        $this->isReferralStillAvailableSpecification = $isReferralStillAvailableSpecification;
    }

    /**
     * @throws ReferralNotFoundForInviteeEmail
     * @throws ReferralAlreadyAcceptedException
     * @throws InviteeAlreadyInvitedException
     */
    public function __invoke(
        AcceptInviteInput $input,
        AcceptInvitePresenterInterface $presenter
    ): void {
        $email = $input->getInvitee()->getEmail();
        $referralModel = $this->referralRepository->findByInviteeEmail($email);
        if (!$referralModel) {
            throw new ReferralNotFoundForInviteeEmail($email);
        }

        if (!($this->isReferralStillAvailableSpecification)($referralModel)) {
            throw new ReferralAlreadyAcceptedException($referralModel);
        }

        $referralModel->setInvitee($input->getInvitee());
        $referralModel = $this->referralRepository->save($referralModel);

        $presenter->present(new AcceptInviteOutput($referralModel));
    }
}
