<?php

declare(strict_types=1);

namespace Dedi\SyliusCustomerReferralPlugin\Domain\Referral\UseCase\Invite;

use Dedi\SyliusCustomerReferralPlugin\Domain\Common\Email\Email;
use Dedi\SyliusCustomerReferralPlugin\Domain\Common\Email\EmailSenderInterface;
use Dedi\SyliusCustomerReferralPlugin\Domain\Referral\DTO\ReferralDTO;
use Dedi\SyliusCustomerReferralPlugin\Domain\Referral\Exception\InviteeAlreadyInvitedException;
use Dedi\SyliusCustomerReferralPlugin\Domain\Referral\Factory\ReferralFactoryInterface;
use Dedi\SyliusCustomerReferralPlugin\Domain\Referral\Repository\ReferralRepositoryInterface;

class Invite
{
    /** @var ReferralFactoryInterface */
    private $referralFactory;

    /** @var ReferralRepositoryInterface */
    private $referralRepository;

    /** @var EmailSenderInterface */
    private $emailSender;

    public function __construct(
        ReferralFactoryInterface $referralFactory,
        ReferralRepositoryInterface $referralRepository,
        EmailSenderInterface $emailSender
    ) {
        $this->referralFactory = $referralFactory;
        $this->referralRepository = $referralRepository;
        $this->emailSender = $emailSender;
    }

    public function __invoke(
        InviteInput $input,
        InvitePresenterInterface $presenter
    ): void {
        $referralDTOs = $input->toReferralDTOs();
        $newReferralModels = [];
        $alreadyInvitedReferralModels = [];

        /** @var ReferralDTO $referralDTO */
        foreach ($referralDTOs as $referralDTO) {
            $referralModel = $this->referralFactory->createReferral($referralDTO);

            try {
                $referralModel = $this->referralRepository->save($referralModel);

                $this->emailSender->send(new Email('invite', [$referralDTO->getInviteeEmail()], [
                    'referral' => $referralModel,
                    'message' => $input->getMessage(),
                ]));

                $newReferralModels[] = $referralModel;
            } catch (InviteeAlreadyInvitedException $e) {
                $alreadyInvitedReferralModels[] = $referralModel;
            }
        }

        $presenter->present(new InviteOutput($newReferralModels, $alreadyInvitedReferralModels));
    }
}
